<?php
// $Id: weblinks.user.inc,v 1.2.2.1 2008/08/01 14:13:26 rmiddle Exp $
/**
 * @file
 * Displays a Drupal page containing weblinks submitted by a given user.
 */
function weblinks_user_page($account) {
  global $user;

  // Node admin style brief look at user weblinks, including pub status
  $output = drupal_get_form('weblinks_filter_form', $form);

  // @@@ Bdragon: No!
  if ($_POST['operation'] == 'delete' && $_POST['nodes']) {
    return drupal_get_form('node_multiple_delete_confirm');
  }
  elseif ($_POST['operation'] == 'unpublish' && $_POST['nodes']) {
    node_operations_unpublish($_POST['nodes']);
  }

  // Call the form first, to allow for the form_values array to be populated.
  $output .= drupal_get_form('weblinks_user_form', $form);
  return $output;
}

function weblinks_user_form() {
  global $user;
  $filter = weblinks_build_filter_query();
  $vocid = _weblinks_get_vocid();

  // @@@ Bdragon: Danger, will robinson!
  $result = db_query('SELECT n.* FROM {node} n '. $filter['join'] .
    ' INNER JOIN {users} u ON n.uid = '. $user->uid .
    ' INNER JOIN {term_node} tn ON tn.nid = n.nid'.
    ' INNER JOIN {term_data} t ON t.tid = tn.tid'.
    ' '. $filter['where'] .' AND t.vid = '. $vocid .
    ' ORDER BY n.changed DESC', $filter['args']);

  $destination = drupal_get_destination();
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $options = array();
  $weblinks_options = array('delete', 'unpublish'); // Should we allow "unpublish"?
  $node_ops = module_invoke_all('node_operations');
  foreach ($node_ops as $operation => $array) {
    if (in_array($operation, $weblinks_options)) {
      $options[$operation] = $array['label'];
    }
  }
  $form['options']['operation'] = array('#type' => 'select', '#options' => $options,  '#default_value' => 'approve');
  $form['options']['submit'] = array('#type' => 'submit', '#value' => t('Update'));

  while ($node = db_fetch_object($result)) {
    $nodes[$node->nid] = '';
    $form['title'][$node->nid] = array('#value' => l($node->title, 'node/'. $node->nid) .' '. theme('mark', node_mark($node->nid, $node->changed)));
    $form['status'][$node->nid] =  array('#value' =>  ($node->status ? t('published') : t('not published')));
    $form['operations'][$node->nid] = array('#value' => l(t('edit'), 'node/'. $node->nid .'/edit', array(), $destination));
  }
  $form['nodes'] = array('#type' => 'checkboxes', '#options' => $nodes);
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));

  return $form;
}

/**
 * Theme node administration overview.
 */
function theme_weblinks_user_form($form) {
  // Overview table:
  $header = array(NULL, t('Title'), t('Status'), t('Operations'));
  $rows =array();

  $output .= drupal_render($form['options']);
  if (isset($form['title']) && is_array($form['title'])) {
    foreach (element_children($form['title']) as $key) {
      $rows[] = array(
        drupal_render($form['nodes'][$key]),
        drupal_render($form['title'][$key]),
        drupal_render($form['status'][$key]),
        drupal_render($form['operations'][$key]),
        );
    }

  }
  else  {
    $rows[] = array(array('data' => t('No posts available.'), 'colspan' => '6'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Return form for node administration filters.
 */
function weblinks_filter_form() {
  $session = &$_SESSION['weblinks_overview_filter'];
  $session = is_array($session) ? $session : array();
  $filters = weblinks_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only items where'),
    '#theme' => 'node_filters',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter; //@@@ Bdragon: Uh, $k => $v syntax on previous line anyone?
    if ($type == 'category') {
      // Load term name from DB rather than search and parse options array.
      $value = taxonomy_get_term($value);
      $value = $value->name;
    }
    else {
      $value = $filters[$type]['options'][$value];
    }
    $string = ($i++ ? '<em>and</em> where <strong>%a</strong> is <strong>%b</strong>' : '<strong>%a</strong> is <strong>%b</strong>');
    $form['filters']['current'][] = array('#value' => $string, array('%a' => $filters[$type]['title'] , '%b' => $value));
  }

  foreach ($filters as $key => $filter) {
    $names[$key] = $filter['title'];
    $form['filters']['status'][$key] = array('#type' => 'select', '#options' => $filter['options']);
  }
  $form['filters']['filter'] = array('#type' => 'radios', '#options' => $names, '#default_value' => 'status');
  $form['filters']['buttons']['submit'] = array('#type' => 'submit', '#value' => (count($session) ? t('Refine') : t('Filter')));
  if (count($session)) {
    $form['filters']['buttons']['undo'] = array('#type' => 'submit', '#value' => t('Undo'));
    $form['filters']['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset'));
  }

  return $form;
}

/**
 * List bw administration filters that can be applied.
 */
function weblinks_filters() {
  // @@@ Bdragon: Skipping looking at this before I learn a bit more about the admin filter page...
  // Regular filters
  $filters['status'] = array('title' => t('status'),
    'options' => array('status-1'   => t('published'),     'status-0' => t('not published'),
                       'moderate-1' => t('in moderation'), 'moderate-0' => t('not in moderation'),
                       'promote-1'  => t('promoted'),      'promote-0' => t('not promoted'),
                       'sticky-1'   => t('sticky'),        'sticky-0' => t('not sticky')));
   // The taxonomy filter
  if ($taxonomy = _weblinks_parent_select($edit['tid'], t('Parent'), 'weblinks')) {
    $filters['category'] = array('title' => t('category'), 'options' => $taxonomy['#options']);
  }

  return $filters;
}

/**
 * Build query for bw administration filters based on session.
 */
function weblinks_build_filter_query() {
  $filters = weblinks_filters();

  // Build query
  $where = $args = array();
  $join = '';
  foreach ($_SESSION['weblinks_overview_filter'] as $index => $filter) {
    list($key, $value) = $filter;
    switch ($key) {
      case 'status':
        // Note: no exploitable hole as $key/$value have already been checked when submitted
        list($key, $value) = explode('-', $value, 2);
        $where[] = 'n.'. $key .' = %d';
        break;
      case 'category':
        $table = "tn$index";
        $make_where = "($table.tid = %d";
        $join .= "INNER JOIN {term_node} $table ON n.nid = $table.nid ";

        // Search recursively for the children of this term
        $children = taxonomy_get_children($_SESSION['weblinks_overview_filter'][$index][1], _weblinks_get_vocid());
        $i = 0;
        foreach ($children as $child) {
          $i++;
          //$table = "tn".$index.$i;
          $make_where .= " OR $table.tid = ". $child->tid ." ";
        }
        $make_where .= ")";
        $where[] = $make_where;
        break;
      case 'type':
        $where[] = "n.type = '%s'";
    }
    $args[] = $value;
  }
  $where = count($where) ? 'WHERE '. implode(' AND ', $where) : '';

  return array('where' => $where, 'join' => $join, 'args' => $args);
}

/**
 * Process result from bw administration filter form.
 */
function weblinks_filter_form_submit() {
  global $_weblinks_form_state;
//  $op = $_POST['op'];
  $op =  $form_state['values']['op'];
  $filters = weblinks_filters();
  switch ($op) {
    case t('Filter'):
    case t('Refine'):
      if (isset($_weblinks_form_state['values']['filter'])) {
        $filter = $_weblinks_form_state['values']['filter'];

        // Flatten the options array to accommodate hierarchical/nested options.
        $flat_options = form_options_flatten($filters[$filter]['options']);

        if (isset($flat_options[$_weblinks_form_state['values'][$filter]])) {
          $_SESSION['weblinks_overview_filter'][] = array($filter, $form_values[$filter]);
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['weblinks_overview_filter']);
      break;
    case t('Reset'):
      $_SESSION['weblinks_overview_filter'] = array();
      break;
  }
}

/**
 * Returns a select box for available parent terms.
 *
 * @param $tid ID of the term which is being added or edited
 * @param $title Title to display the select box with
 * @param $child_type Whether the child is weblink or category
 */
function _weblinks_parent_select($tid, $title, $child_type) {
  $parents = taxonomy_get_parents($tid);
  if ($parents) {
    $parent = array_shift($parents);
    $parent = $parent->tid;
  }
  else {
    $parent = 0;
  }

  $children = taxonomy_get_tree(_weblinks_get_vocid(), $tid);

  // A term can't be the child of itself, nor of its children.
  foreach ($children as $child) {
    $exclude[] = $child->tid;
  }
  $exclude[] = $tid;

  // If taxonomy_access is enabled, there may be group permissions to check
  $include = _weblinks_get_access();

  $tree = taxonomy_get_tree(_weblinks_get_vocid());
  if (user_access('administer weblinks')) {
    $options[0] = '<'. t('root') .'>';
  }
  if ($tree) {
    foreach ($tree as $term) {
      if (!in_array($term->tid, $exclude)) {
        // If user doesn't have admin priveledges, then user must have a specific role to be here
        if ((user_access('administer weblinks')) || (user_access('edit group weblinks') && (in_array($term->tid, $include)))) {
          //$options[$term->tid] = _taxonomy_depth($term->depth) . $term->name;
          $options[$term->tid] = str_repeat('-', $term->depth) . $term->name;
        }
      }
    }
  }
  if ($child_type == 'container') {
    $description = t('Containers are usually placed at the top (root) level of your web link groups but you can also place a container inside a parent container or group.');
  }
  else if ($child_type == 'forum') {
    $description = t('You may place your web link inside a parent container or group, or at the top (root) level of your groups.');
  }

  return array('#type' => 'select', '#title' => $title, '#default_value' => $parent, '#options' => $options, '#description' => $description, '#required' => TRUE);
}

/**
 * Returns a confirmation page for deleting a weblinks taxonomy term.
 *
 * @param $tid ID of the term to be deleted
 */
function weblinks_confirm_delete($tid) {
  $term = taxonomy_get_term($tid);

  $form['tid'] = array('#type' => 'hidden', '#value' => $tid);
  $form['name'] = array('#type' => 'hidden', '#value' => $term->name);

  return confirm_form($form, t('Are you sure you want to delete the group %name?', array('%name' => $term->name)), 'admin/content/weblinks', t('Deleting a weblink or container will delete <b>all sub-groups and associated links as well</b>. This action cannot be undone.'), t('Delete'), t('Cancel'));
}

/**
 * Implementation of forms api _submit call. Deletes a link category after confirmation.
 */
function weblinks_confirm_delete_submit($form, &$form_state) {
  // @@@ Bdragon: Replace this with a call to drupal_confirm_form!
  taxonomy_del_term($form_state['values']['tid']);
  drupal_set_message(t('The group %term and all sub-groups and associated links have been deleted.', array('%term' => theme('placeholder', $form_state['values']['name']))));
  $form_state['redirect'] = 'admin/content/weblinks';
  return;
}
